import 'package:vivanta_connect_flutter/helpers/helpers.dart';
import 'package:vivanta_connect_flutter/models/apple_health_element.dart';
import 'package:vivanta_connect_flutter/services/database.dart';
import 'package:vivanta_connect_flutter/services/healthkit.dart';
import 'package:vivanta_connect_flutter/services/http.dart';
import 'package:vivanta_connect_flutter/services/preferences.dart';

class VivantaSync {
  final String apiKey;
  final String customerId;
  final String externalUserId;
  late final Database database;
  late final Http http;
  late final Preferences preferences;

  VivantaSync({
    required this.apiKey,
    required this.customerId,
    required this.externalUserId,
  }) {
    http = Http();
    database = Database();
    preferences = Preferences();
  }

  Future<void> executeAll() async {
    await database.init();

    await executeAppleHealth(database, preferences, http);

    return;
  }

  Future<void> executeAppleHealth(
    Database database,
    Preferences preferences,
    Http http,
  ) async {
    if (await preferences.getAppleHealthAccessGranted()) {
      await _getAppleHealthData(preferences, http);
      await _syncAppleHealthData(database);
    }
  }

  Future<void> _getAppleHealthData(Preferences preferences, Http http) async {
    final healthkit = Healthkit();

    final client = await http.me(apiKey, customerId);
    final healthDataTypes = getHealthDataTypesFromClient(client);

    final lastSync = await preferences.getAppleHealthSyncDate();
    final todaySync = DateTime.now();
    final values = await healthkit.getDataPoint(
      healthDataTypes,
      lastSync,
      todaySync,
    );
    for (final value in values) {
      final element = AppleHealthElement(
        id: value.sourceId,
        value: value.toJson().toString(),
        date: DateTime.now().toIso8601String(),
        synced: false,
      );
      await database.saveAppleHealthElement(element);
    }
    await preferences.setAppleHealthSyncDate(todaySync);
  }

  Future<void> _syncAppleHealthData(Database database) async {
    final appleHealthElements = database.getUnsyncedAppleHealthElements();
    for (final element in appleHealthElements) {
      if (element.value != null) {
        final result = await _uploadAppleHealthData(http, element.value!);
        if (result) {
          element.synced = true;
          element.syncedDate = DateTime.now().toIso8601String();
          await database.updateAppleHealthElement(element);
        }
      }
    }
  }

  Future<bool> _uploadAppleHealthData(Http http, String value) {
    return http.uploadAppleHealthData(
      apiKey,
      customerId,
      externalUserId,
      value,
    );
  }
}
